from django.db import models

class Project(models.Model):
	project = models.CharField(max_length=100, help_text="Defines the Project, on which the Test Case belongs.")

	def __unicode__(self):
		return self.project


class Team(models.Model):
	team = models.CharField(max_length=100, help_text="Defines the Team, on which the Test Case belongs.")
	
	def __unicode__(self):
		return self.team


class State(models.Model):
	state = models.CharField(max_length=100, help_text="State of the Test Case: Passed, Passed W/E, Failed, Blocked.")
	color = models.CharField(max_length=100, help_text="This field accepts HTML HEX color schemes: #000000, #ffffff")
	exp = models.CharField(max_length=10, help_text="This field is for the pie chart presentation: Values should be close to 0.01 - 0.09.")
	
	def __unicode__(self):
		return self.state


class Report(models.Model):
        title = models.CharField(max_length=100, help_text="This field represents the title of the Test Case.")
        description = models.TextField(help_text="This field describes the content of each Test Case.")
       	configuration = models.TextField(help_text="This field describes the test configuration and the setup for the test case.")
	project = models.ForeignKey(Project)
	team = models.ForeignKey(Team)
	tid = models.CharField(max_length=130, help_text="Test Case ID, provided by QA.", unique=True)
	state = models.ForeignKey(State)
 	owner = models.CharField(max_length=130, help_text="Defines the QA Owner of the Test Case.") 
	created_date = models.DateTimeField(auto_now_add=True)
	updated_date = models.DateTimeField(auto_now_add=True)

	def __unicode__(self):
                return self.title


class AutoCompleteKeyword(models.Model):
	keyword = models.CharField(max_length=100, help_text="Keywords are used for autocompletion", unique=True)

	def __unicode__(self):
		return self.keyword

