# Create your views here.
# -*- coding: UTF-8 -*-

from datetime import datetime
from django import forms
from QAWeb.models import *
from QAWeb.index import complete_indexer
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.http import Http404, HttpResponse
from django.shortcuts import render_to_response
from django.utils.safestring import mark_safe
from django.contrib.syndication.views import Feed
from django.utils import simplejson
from django.conf import settings

# Imports for the Graphs
import os
os.environ['HOME']= settings.HOME

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from pylab import *

MODEL_MAP = { 'report': Report }
MODEL_CHOICES = [('', 'all')] + zip(MODEL_MAP.keys(), MODEL_MAP.keys())


class LatestEntriesFeed(Feed):
	title = "QA Web Portal - Last Test Cases"
	link = "/"
	description = "Recent Test Cases"

	def items(self):
		return Report.objects.order_by('-updated_date')[:50]

	def item_title(self, item):
		return item.title

	def item_description(self, item):
		return item.description

	def item_link(self, item):
		return	'/preview/?id=%s' % (item.id)
		
def index(request):
	latestreports = Report.objects.all().order_by('?')[:15]
	allreports = Report.objects.all().count()
	allteams = Team.objects.all().count()
	allprojects = Project.objects.all().count()
	return render_to_response('QAWeb/index.html', {'latestreports':latestreports, 'allreports':allreports, 'allteams':allteams, 'allprojects':allprojects})

# This method contains deep hacks for making statistics updatable on the fly
def teamview(request):
	teams = Team.objects.all()
	states = State.objects.all()
	stats = []
	labels = []
	colors = []
	total = []
	new = []

        for j in range(len(teams)):
		for i in range(len(states)):
	            new.append(0)
		stats.append(new)
		new = []
	
	directory = settings.GRAPHS_DIR
	figtype = '.png'
	counter = 0 		

	for t in teams:
		fracs = []
                labels = []
		colors = []
		colors_p = []
                labels_p = []
                explode_p = []
		added = 0			
		counter2 =0

		fig = plt.figure(1, figsize=(6,6))
		for s in states:
			colors.append(s.color)
                        labels.append(s.state)
			if Report.objects.all().filter(team_id=t.id, state_id=s.id).count() > 0:
				added += Report.objects.all().filter(team_id=t.id, state_id=s.id).count() 
				colors_p.append(s.color)
       				labels_p.append(s.state)
       				explode_p.append(float(s.exp))
				stats[counter][counter2] = Report.objects.all().filter(team_id=t.id, state_id=s.id).count()
				fracs.append(stats[counter][counter2])
			else:
				stats[counter][counter2] = 0
			counter2 += 1
		pie(fracs, colors=colors_p, explode=explode_p, labels=labels_p, autopct='%1.1f%%', shadow=True)
		name = t.team
		fig.savefig(directory + name + figtype)
		close()
		total.append(added)
		counter += 1
	if not teams:
		teams = ['No teams in the systems']
	return render_to_response('QAWeb/teamview.html', {'teams':teams, 'stats':stats, 'labels':labels, 'total':total, 'colors':colors})

# This method contains deep hacks for making statistics updatable on the fly
def projectview(request):
	projects = Project.objects.all()
	states = State.objects.all()
	stats = []
	labels = []
	colors = []
	total = []
	new = []

        for j in range(len(projects)):
		for i in range(len(states)):
	            new.append(0)
		stats.append(new)
		new = []
	
	directory = settings.GRAPHS_DIR
	figtype = '.png'
	counter = 0 		

	for p in projects:
		fracs = []
		colors = []
		labels = []
		colors_p = []
		labels_p = []
		explode_p = []
		added = 0			
		counter2 =0

		fig = plt.figure(1, figsize=(6,6))
		for s in states:
                        colors.append(s.color)
                        labels.append(s.state)
			if Report.objects.all().filter(project_id=p.id, state_id=s.id).count() > 0:
				added += Report.objects.all().filter(project_id=p.id, state_id=s.id).count() 
				colors_p.append(s.color)
       				labels_p.append(s.state)
       				explode_p.append(float(s.exp))
				stats[counter][counter2] = Report.objects.all().filter(project_id=p.id, state_id=s.id).count()
				fracs.append(stats[counter][counter2])
			else:
				stats[counter][counter2] = 0
			counter2 += 1
		pie(fracs, colors=colors_p, explode=explode_p, labels=labels_p, autopct='%1.1f%%', shadow=True)
		name = p.project
		fig.savefig(directory + name + figtype)
		close()
		total.append(added)
		counter += 1
	if not projects:
		projects = ['No projects in the system']
	return render_to_response('QAWeb/projectview.html', {'projects':projects, 'stats':stats, 'labels':labels, 'total':total, 'colors':colors})

def overview(request):
	total = Report.objects.all().count()
	states = State.objects.all()
	stats = []
	fracs = []
  	colors = []
        labels = []
	colors_p = []
	labels_p = []
	explode_p = []	

	directory = settings.GRAPHS_DIR
	fig = plt.figure(1, figsize=(6,6))
	for s in states:
                colors.append(s.color)
                labels.append(s.state)
		if Report.objects.all().filter(state_id=s.id).count() > 0:
			colors_p.append(s.color)
	                labels_p.append(s.state)
        	        explode_p.append(float(s.exp))
			fracs.append(Report.objects.all().filter(state_id=s.id).count())
			stats.append(Report.objects.all().filter(state_id=s.id).count())
		else:
			stats.append(0)
	pie(fracs, colors=colors_p, explode=explode_p, labels=labels_p, autopct='%1.1f%%', shadow=True)
	name = "Overview.png"
	fig.savefig(directory + name)
	close()
	return render_to_response('QAWeb/overview.html', {'stats':stats, 'labels':labels, 'total':total, 'colors':colors})

def howto(request):
	teams = Team.objects.all()
	states = State.objects.all()
	projects = Project.objects.all()
	return render_to_response('QAWeb/howto.html', {'teams':teams, 'projects':projects, 'states':states})

def about(request):
	return render_to_response('QAWeb/about.html')

def preview(request):
	if request.GET:
		by_id = request.GET["id"] 
		report = Report.objects.get(id=by_id)
		return render_to_response('QAWeb/preview.html', {'report': report})
	else:
		return render_to_response('QAWeb/preview.html')


class SearchForm(forms.Form):
    query = forms.CharField(required=True)
    model = forms.ChoiceField(choices=MODEL_CHOICES, required=False)


def search(request):
    results = []
    latestreports = Report.objects.all().order_by('-updated_date')[:15]
    allreports = Report.objects.all().count()
    allteams = Team.objects.all().count()
    allprojects = Project.objects.all().count()
    if request.GET:
        form = SearchForm(request.GET)
        if form.is_valid():
                query = form.cleaned_data['query']
                model = MODEL_MAP.get(form.cleaned_data['model'])
		q_keeper = query 

                if not model:
                        indexer = complete_indexer
                else:
                        indexer = model.indexer

                paginator = Paginator(indexer.search(query), 15)

                try:
                        svalue = int(request.GET.get('page', 1))
                except:
                        svalue = int(1)
                
		try:
                        results = paginator.page(svalue)
                except PageNotAnInteger:
                        results = paginator.page(int(1))
                except EmptyPage:
                        results = paginator.page(int(1))

                if not paginator.count:
                        return render_to_response('QAWeb/search.html', {'results': results, 'query': query }) 
                else:
			if len(query) > 4:
				newkeyword, created = AutoCompleteKeyword.objects.get_or_create(keyword=query)
                        return render_to_response('QAWeb/search.html', {'results': results, 'query': query })
    else:
        form = SearchForm()
    return render_to_response('QAWeb/index.html', {'latestreports':latestreports, 'allreports':allreports, 'allteams':allteams, 'allprojects':allprojects})


def qa_jpkeys(request):
    if 'term' in request.GET:
        q = request.GET['term']
        keywords = AutoCompleteKeyword.objects.order_by('?').filter(keyword__istartswith=q)[:8]
        return HttpResponse( simplejson.dumps( [ k.keyword for k in keywords ] ) )
    return HttpResponse()
 
